#define GINT_NEED_VRAM
#include <gint/display.h>
#include <stdint.h>
#include <fxengine/fxengine.h>
#ifdef USE_LIBPROF 
#include <libprof.h>
#endif
#include <gint/std/stdlib.h>
#include <stdbool.h>

// FPS count
#ifdef USE_LIBPROF 
static uint32_t frame_interval_min=1000000;
static uint32_t frame_interval_max=1;
static uint32_t fps=0;
#endif
uint32_t fe_get_fps()
{return fps;}

// Triple buffering
#define BUFFER_SIZE 256
static uint32_t triple_buffering_area[2*BUFFER_SIZE]; // 2 buffers supplémentaires
static uint32_t * const temp_buffers[2] = {&triple_buffering_area[0], &triple_buffering_area[BUFFER_SIZE]};
static uint32_t * gint_vram=0;
static uint32_t working_buffer=0;

static void check_vram()
{
    if (gint_vram==0)
        gint_vram=vram;
}

void fe_callback_start()
{
    vram=gint_vram;
    memcpy(gint_vram, temp_buffers[1-working_buffer], sizeof(uint32_t)*BUFFER_SIZE);
}

void fe_callback_end()
{
    vram=temp_buffers[working_buffer];
}

void fe_draw()
{
    memcpy(gint_vram, temp_buffers[1-working_buffer], sizeof(uint32_t)*BUFFER_SIZE);
}

void switch_frame()
{
    working_buffer=1-working_buffer;
    vram=temp_buffers[working_buffer];
    dclear(C_WHITE);
}

static volatile bool quit=true;

void fe_load()
{
    check_vram();
    quit=false;
    while (1)
    {
        fe_zbuffer_clear();
        render();
        switch_frame();
        if (quit)
            break;
    }
    // Free some malloc
    //// as object list
}

void fe_quit()
{
    quit=1;
}

/*

#include <fxengine/zbuffer.h>

#include <gint/std/stdio.h>
#include <gint/display.h>

#include <gint/exc.h>
#include <gint/defs/attributes.h>

static void * fe_rendering_list;

#ifdef USE_LIBPROF
#include <libprof.h>
static uint32_t frame_interval_min=1000000;
static uint32_t frame_interval_max=1;
static uint32_t fps=0;
#endif

// Double buffering
static uint32_t * vram1 = vram;
static uint32_t vram2[1024/sizeof(uint32_t)];
static uint32_t * vrams[2]={vram1,vram2};
static uint32_t drawing_vram=1;

static void switch_vram()
{
    vram=vrams[drawing_vram];
    drawing_vram=1-drawing_vram;
}

void fe_set_user_mode();
void fe_set_working_mode();


int32_t fe_get_fps()
{
#ifdef USE_LIBPROF
    return fps;
#else
    return -1;
#endif
}


void fe_display(const uint32_t libprof_channel)
{
    fe_update(libprof_channel);
    dupdate();
    dclear(C_WHITE);
}


void fe_update(const uint32_t libprof_channel)
{
#ifdef USE_LIBPROF
    if (prof_elapsed)
    {
        prof_leave(libprof_channel);
        uint32_t frame_interval = prof_time(libprof_channel);
        //sleep_us(0, MINIMUM_FRAME_DELAY-frame_interval);
        if (frame_interval<frame_interval_min)
            frame_interval_min = frame_interval;
        if (frame_interval>frame_interval_max)
            frame_interval_max = frame_interval;
        fps=1000000/frame_interval;
    }
    else
    {
        prof_init(libprof_channel+1, 0);
    }
    prof_clear(libprof_channel);
    prof_enter(libprof_channel);
#endif
    switch_vram();
    fe_zbuffer_clear();
}

*/